function [xo,Ot,nS,it]=cgrad(S,x0,ip,G,method,Lb,Ub,problem,tol,mxit)
%   Unconstrained optimization using conjugate gradients (Fletcher-Reeves).
%
%   [xo,Ot,nS]=cgrad(S,x0,ip,G,method,Lb,Ub,problem,tol,mxit)
%
%   S: objective function
%   x0: initial point
%   ip: (0) no plot (default), (>0) plot figure ip with pause, (<0) plot figure ip
%   G: gradient vector function
%   method: line-search method: (0) Coggins (default), (1): Golden Section
%   Lb, Ub: lower and upper bound vectors to plot (default = x0*(1+/-2))
%   problem: (-1): minimum (default), (1): maximum
%   tol: tolerance (default = 1e-4)
%   mxit: maximum number of iterations (default = 50*(1+4*~(ip>0)))
%   xo: optimal point
%   Ot: optimal value of S
%   nS: number of objective function evaluations

%   Copyright (c) 2001 by LASIM-DEQUI-UFRGS
%   $Revision: 1.0 $  $Date: 2001/07/16 08:10:12 $
%   Argimiro R. Secchi (arge@enq.ufrgs.br)

 if nargin < 2,
   error('cgrad requires two input arguments');
 end
 if nargin < 3 | isempty(ip),
   ip=0;
 end
 if nargin < 4 | isempty(G),
   grad=0;
 else
   grad=1;
 end
 if nargin < 5 | isempty(method),
   method=0;
 end
 if nargin < 6 | isempty(Lb),
   Lb=-x0-~x0;
 end
 if nargin < 7 | isempty(Ub),
   Ub=2*x0+~x0;
 end
 if nargin < 8 | isempty(problem),
   problem=-1;
 end
 if nargin < 9 | isempty(tol),
   tol=1e-4;
 end
 if nargin < 10 | isempty(mxit),
   mxit=50*(1+4*~(ip>0));
 end

 x0=x0(:);
 y0=feval(S,x0)*problem;
 n=size(x0,1);
 ips=ip;
 d=1e-5*abs(x0)+1e-5*ones(n,1);
 CHG=0.01*d;
 gr=d;
 
 if ip & n == 2,
   figure(abs(ip));
   [X1,X2]=meshgrid(Lb(1):(Ub(1)-Lb(1))/20:Ub(1),Lb(2):(Ub(2)-Lb(2))/20:Ub(2));
   [n1,n2]=size(X1);
   f=zeros(n1,n2);
   for i=1:n1,
    for j=1:n2,
      f(i,j)=feval(S,[X1(i,j);X2(i,j)]);
    end
   end
   mxf=max(max(f));
   mnf=min(min(f));
   df=mnf+(mxf-mnf)*(2.^(([0:10]/10).^2)-1);
   [v,h]=contour(X1,X2,f,df); hold on;
%   clabel(v,h);
   h1=plot(x0(1),x0(2),'ro');
   legend(h1,'start point');

   if ip > 0,
     ips=ip+1;
     disp('Pause: hit any key to continue...'); pause;
   else
     ips=ip-1;
   end
 end
 
 xo=x0;
 yo=y0;
 it=0;
 nS=1;
 k=0;
 
 while it < mxit,
   if ~grad,
     y=yo;
     % Finite difference perturbation levels
     % First check perturbation level is not less than search direction.
     j=find(10*abs(CHG)>abs(d));
     CHG(j)=-0.1*d(j);
     % Ensure within user-defined limits
     for i=1:n
        xo(i)=xo(i)+CHG(i);
        yo=feval(S,xo)*problem;
        gr(i)=(yo-y)/CHG(i);
        if yo > y,
          y=yo;
        else
          xo(i)=xo(i)-CHG(i);
        end
     end
     % Try to set difference to 1e-8 for next iteration
     % Add eps for machines that can't handle divide by zero.
     CHG=1e-8./(gr+eps); 
     yo=y;
     nS=nS+n;
   else
     gr=feval(G,xo)*problem;
     gr=gr(:);
   end

  if it,
    if norm(xo-x0) < tol*(0.1+norm(x0)) & abs(yo-y0) < tol*(0.1+abs(y0)),
      break;
    end
    k=k+1;
    if k == n,
      d=-gr;
      k=0;
    else
      d=d*gr'*gr/(gr0'*gr0)-gr;
    end
  else
    d=-gr;
  end

  it=it+1;
  x0=xo;
  y0=yo;
  gr0=gr;
   
  if method,           % to see the linesearch plot, remove the two 0* below
    [stepsize,xo,yo,nS1]=aurea(S,x0,d/norm(d),0*ips,problem,tol,mxit);
    yo=yo*problem;
  else
    [xo,yo,nS1,stepsize]=coggins(S,x0,d/norm(d),0*ips,problem,tol,mxit);
    yo=yo*problem;
  end

  nS=nS+nS1;
  
  if ip & n == 2,
    plot([x0(1) xo(1)],[x0(2) xo(2)],'r');
    if ip > 0,
      disp('Pause: hit any key to continue...'); pause;
    end
  end 
 end
 
 Ot=yo*problem;
 
 if it == mxit,
   disp('Warning Cgrad: reached maximum number of iterations!');
 elseif ip & n == 2,
   h2=plot(xo(1),xo(2),'r*');
   legend([h1,h2],'start point','optimum');
 end
